/*
   +----------------------------------------------------------------------+
   | PHP Version 5                                                        |
   +----------------------------------------------------------------------+
   | Copyright (c) 1997-2010 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
   | available through the world-wide-web at the following url:           |
   | http://www.php.net/license/3_01.txt                                  |
   | If you did not receive a copy of the PHP license and are unable to   |
   | obtain it through the world-wide-web, please send a note to          |
   | license@php.net so we can mail you a copy immediately.               |
   +----------------------------------------------------------------------+
   | Author: Melanie Rhianna Lewis <cyberspice@php.net>                   |
   +----------------------------------------------------------------------+
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "php.h"
#include "php_main.h"
#include "php_globals.h"
#include "zend_objects.h"
#include "zend_exceptions.h"
#include "ext/standard/info.h"
#include "php_framegrab.h"
#include "php_framegrab_device.h"
#include "php_framegrab_image.h"

#ifndef Z_SET_REFCOUNT_P
# define Z_SET_REFCOUNT_P(pz, cnt) (pz)->refcount = cnt;
#endif

#ifndef Z_UNSET_ISREF_P
# define Z_UNSET_ISREF_P(pz) (pz)->is_ref = 0;
#endif

/*
 * Macro that is like ZVAL_LONG, ZVAL_BOOL etc but for objects.
 * Takes a zval pointer and class entry pointer.
 */
#define ZVAL_OBJECT(z, ce)  { \
		(z)->type = IS_OBJECT; \
		object_init_ex((z), (ce)); \
		Z_UNSET_ISREF_P(z); \
		Z_SET_REFCOUNT_P(z, 1); \
	}

/*
 * Macro that is like RETVAL_LONG, RETVAL_BOOL etc but for objects.
 * Takes a class entry pointer.
 */
#define RETVAL_OBJECT(ce) ZVAL_OBJECT(return_value, ce)

/* The class entry pointers */
zend_class_entry *php_framegrab_device_exception_class_entry;
zend_class_entry *php_framegrab_device_sc_entry;
zend_class_entry *php_framegrab_image_sc_entry;

/* The object handlers */
static zend_object_handlers framegrab_device_object_handlers;
static zend_object_handlers framegrab_image_object_handlers;

typedef struct _php_framegrab_image_obj {
	zend_object zo;
	php_framegrab_image_data image;

} php_framegrab_image_obj;

/* {{{ proto FrameGrabImage::__construct()
 * Constructs a new FrameGrabImage object. */
PHP_METHOD(framegrabimage, __construct) {
	php_framegrab_image_obj *intern;
#if ZEND_MODULE_API_NO > 20060613
	zend_error_handling error_handling;

	zend_replace_error_handling(EH_THROW, php_framegrab_device_exception_class_entry, &error_handling TSRMLS_CC);
#else	
	php_set_error_handling(EH_THROW, php_framegrab_device_exception_class_entry TSRMLS_CC);
#endif

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "") == FAILURE) {
#if ZEND_MODULE_API_NO > 20060613 
	zend_restore_error_handling(&error_handling TSRMLS_CC);
#else
	php_set_error_handling(EH_NORMAL, NULL TSRMLS_CC);
#endif
		return;
	}

	intern = (php_framegrab_image_obj *)zend_object_store_get_object(getThis() TSRMLS_CC);
	php_framegrab_image_data_init(&(intern->image));

#if ZEND_MODULE_API_NO > 20060613 
	zend_restore_error_handling(&error_handling TSRMLS_CC);
#else
	php_set_error_handling(EH_NORMAL, NULL TSRMLS_CC);
#endif
}
/* }}} */

/* {{{ proto FrameGrabImage::__toString()
 * Outputs image meta data as a string. */
PHP_METHOD(framegrabimage, __toString) {
	php_framegrab_image_obj *intern;
	char *buffer;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "") == FAILURE) {
		RETURN_FALSE
	}

	intern = (php_framegrab_image_obj *)zend_object_store_get_object(getThis() TSRMLS_CC);
	if (NULL == (buffer = php_framegrab_image_data_to_string(&(intern->image)))) {
		RETURN_FALSE
	}

	ZVAL_STRING(return_value, buffer, 1)
	free(buffer);
	return;
}
/* }}} */

/* {{{ proto FrameGrabImage::writeAsPNG(filename)
 * Writes image as PNG to filename. */
PHP_METHOD(framegrabimage, writeAsPNG) {
	char *filename = NULL;
	int filename_len = 0;
	php_framegrab_image_obj *intern;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &filename, &filename_len) == FAILURE) {
		RETURN_FALSE
	}

	intern = zend_object_store_get_object(getThis() TSRMLS_CC);
	if (php_framegrab_image_write(&(intern->image), filename, FRAMEGRAB_IMAGE_OUTPUT_PNG)) {
		RETURN_TRUE
	} else {
		RETURN_FALSE
	}
}
/* }}} */

/* {{{ proto FrameGrabImage::toStringAsPNG()
 * Outputs image data in PNG format as a string. */
PHP_METHOD(framegrabimage, toStringAsPNG) {
	php_framegrab_image_obj *intern;
	char *buffer;
	int buffer_len;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "") == FAILURE) {
		RETURN_FALSE
	}

	intern = (php_framegrab_image_obj *)zend_object_store_get_object(getThis() TSRMLS_CC);
	if (!php_framegrab_image_to_string(&(intern->image), (FG_UINT8 **)&buffer, &buffer_len, FRAMEGRAB_IMAGE_OUTPUT_PNG)) {
		RETURN_FALSE
	}

	ZVAL_STRINGL(return_value, buffer, buffer_len, 1)
	free(buffer);
	return;
}
/* }}} */

/* {{{ */
static void framegrab_image_object_free(void *object TSRMLS_DC) {
	php_framegrab_image_obj *intern = (php_framegrab_image_obj *)object;

	/* Free up the image data */
	php_framegrab_image_data_destroy(&intern->image);

	zend_object_std_dtor(&intern->zo TSRMLS_CC);
	efree(object);
}
/* }}} */

/* {{{ */
static zend_object_value framegrab_image_object_new_ex(zend_class_entry *class_type, php_framegrab_image_obj **ptr TSRMLS_DC) {
	php_framegrab_image_obj *intern;
	zend_object_value retval;
	zval *tmp;

	intern = emalloc(sizeof(php_framegrab_image_obj));
	memset(intern, 0, sizeof(php_framegrab_image_obj));

	if (ptr) {
		*ptr = intern;
	}

	zend_object_std_init(&intern->zo, class_type TSRMLS_CC);
	zend_hash_copy(intern->zo.properties, &class_type->default_properties, (copy_ctor_func_t) zval_add_ref, (void *) &tmp, sizeof(zval *));

	retval.handle = zend_objects_store_put(intern, NULL, (zend_objects_free_object_storage_t) framegrab_image_object_free, NULL TSRMLS_CC);
	retval.handlers = (zend_object_handlers *) &framegrab_image_object_handlers;
	return retval;
}
/* }}} */

/* {{{ */
static zend_object_value framegrab_image_object_new(zend_class_entry *class_type TSRMLS_DC) {
	return framegrab_image_object_new_ex(class_type, NULL TSRMLS_CC);
}
/* }}} */

/* {{{ */
static zend_object_value framegrab_image_object_clone(zval *this_ptr TSRMLS_DC) {
	php_framegrab_image_obj *new_obj = NULL;
	php_framegrab_image_obj *old_obj = (php_framegrab_image_obj *)zend_object_store_get_object(this_ptr TSRMLS_CC);
	zend_object_value        retval = framegrab_image_object_new_ex(old_obj->zo.ce, &new_obj TSRMLS_CC);

	zend_objects_clone_members(&new_obj->zo, retval, &old_obj->zo, Z_OBJ_HANDLE_P(this_ptr) TSRMLS_CC);

	php_framegrab_image_data_init(&new_obj->image);
	php_framegrab_image_data_clone(&old_obj->image, &new_obj->image);

	return retval;
}
/* }}} */

#define FRAMEGRAB_IMAGE_PROP_NAME_WIDTH              "width"
#define FRAMEGRAB_IMAGE_PROP_NAME_LEN_WIDTH          5
#define FRAMEGRAB_IMAGE_PROP_NAME_HEIGHT             "height"
#define FRAMEGRAB_IMAGE_PROP_NAME_LEN_HEIGHT         6
#define FRAMEGRAB_IMAGE_PROP_NAME_FORMAT             "format"
#define FRAMEGRAB_IMAGE_PROP_NAME_LEN_FORMAT         6
#define FRAMEGRAB_IMAGE_PROP_NAME_BYTES_PER_LINE     "bytes_per_line"
#define FRAMEGRAB_IMAGE_PROP_NAME_LEN_BYTES_PER_LINE 14
#define FRAMEGRAB_IMAGE_PROP_NAME_IMAGE              "image"
#define FRAMEGRAB_IMAGE_PROP_NAME_LEN_IMAGE          5

/* {{{ */
static zval *framegrab_image_object_read_property(zval *object, zval *member, int type TSRMLS_DC) {
	php_framegrab_image_obj *intern;
	zval *retval;
	MAKE_STD_ZVAL(retval);

	intern = (php_framegrab_image_obj *)zend_object_store_get_object(object TSRMLS_CC);
	if (Z_TYPE_P(member) != IS_STRING) {
		ZVAL_NULL(retval);
	} else {
		char *prop_name   = Z_STRVAL_P(member);
		int prop_name_len = Z_STRLEN_P(member);

		if ((FRAMEGRAB_IMAGE_PROP_NAME_LEN_WIDTH == prop_name_len) &&
			!memcmp(prop_name, FRAMEGRAB_IMAGE_PROP_NAME_WIDTH, FRAMEGRAB_IMAGE_PROP_NAME_LEN_WIDTH)) {
			ZVAL_LONG(retval, intern->image.width);
		} else
		if ((FRAMEGRAB_IMAGE_PROP_NAME_LEN_HEIGHT == prop_name_len) &&
			!memcmp(prop_name, FRAMEGRAB_IMAGE_PROP_NAME_HEIGHT, FRAMEGRAB_IMAGE_PROP_NAME_LEN_HEIGHT)) {
			ZVAL_LONG(retval, intern->image.height);
		} else
		if ((FRAMEGRAB_IMAGE_PROP_NAME_LEN_FORMAT == prop_name_len) &&
			!memcmp(prop_name, FRAMEGRAB_IMAGE_PROP_NAME_FORMAT, FRAMEGRAB_IMAGE_PROP_NAME_LEN_FORMAT)) {
			ZVAL_STRING(retval, php_framegrab_image_format_to_string(intern->image.format), 1);
		} else
		if ((FRAMEGRAB_IMAGE_PROP_NAME_LEN_BYTES_PER_LINE == prop_name_len) &&
			!memcmp(prop_name, FRAMEGRAB_IMAGE_PROP_NAME_BYTES_PER_LINE, FRAMEGRAB_IMAGE_PROP_NAME_LEN_BYTES_PER_LINE)) {
			ZVAL_LONG(retval, intern->image.bytes_per_line);
		} else
		if ((FRAMEGRAB_IMAGE_PROP_NAME_LEN_IMAGE == prop_name_len) &&
			!memcmp(prop_name, FRAMEGRAB_IMAGE_PROP_NAME_IMAGE, FRAMEGRAB_IMAGE_PROP_NAME_LEN_IMAGE)) {
			ZVAL_STRINGL(retval, intern->image.image, intern->image.size, 1);
		} else {
			ZVAL_NULL(retval);
		}
	}

	Z_SET_REFCOUNT_P(retval, 0);
	return retval;
}
/* }}} */

static zend_function_entry framegrab_image_funcs[] = {
	PHP_ME(framegrabimage, __construct, NULL, ZEND_ACC_CTOR|ZEND_ACC_PROTECTED)
	PHP_ME(framegrabimage, __toString, NULL, ZEND_ACC_PUBLIC)
	PHP_ME(framegrabimage, writeAsPNG, NULL, ZEND_ACC_PUBLIC)
	PHP_ME(framegrabimage, toStringAsPNG, NULL, ZEND_ACC_PUBLIC)

	/* End of functions */
	{NULL, NULL, NULL}
};

typedef struct _php_framegrab_device_obj {
	zend_object zo;
	php_framegrab_device_data device;

} php_framegrab_device_obj;

/* {{{ proto FrameGrabDevice::__construct()
 * Constructs a new FrameGrabDevice object. */
PHP_METHOD(framegrabdevice, __construct) {
	char *device_name = NULL;
	int device_name_len = 0;
	php_framegrab_device_obj *intern;
	
#if ZEND_MODULE_API_NO > 20060613
	zend_error_handling error_handling;

	zend_replace_error_handling(EH_THROW, php_framegrab_device_exception_class_entry, &error_handling TSRMLS_CC);
#else	
	php_set_error_handling(EH_THROW, php_framegrab_device_exception_class_entry TSRMLS_CC);
#endif

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|s", &device_name, &device_name_len) == FAILURE) {
#if ZEND_MODULE_API_NO > 20060613 
	zend_restore_error_handling(&error_handling TSRMLS_CC);
#else
	php_set_error_handling(EH_NORMAL, NULL TSRMLS_CC);
#endif
		return;
	}
	
#if ZEND_MODULE_API_NO > 20060613 
	zend_restore_error_handling(&error_handling TSRMLS_CC);
#else
	php_set_error_handling(EH_NORMAL, NULL TSRMLS_CC);
#endif

	intern = zend_object_store_get_object(getThis() TSRMLS_CC);
	php_framegrab_device_data_init(&(intern->device));

	if (device_name) {
		if (!php_framegrab_get_device_data(&(intern->device), device_name)) {
			zend_throw_exception(php_framegrab_device_exception_class_entry, "Could not initialise device!", 0 TSRMLS_CC);
			RETURN_FALSE
		}
	} else {
		if (!php_framegrab_get_default_device_data(&(intern->device))) {
			zend_throw_exception(php_framegrab_device_exception_class_entry, "Could not initialise default device!", 0 TSRMLS_CC);
			RETURN_FALSE
		}
	}
}
/* }}} */

/* {{{ proto FrameGrabImage::__toString()
 * Outputs image meta data as a string. */
PHP_METHOD(framegrabdevice, __toString) {
	php_framegrab_device_obj *intern;
	char *buffer;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "") == FAILURE) {
		RETURN_FALSE
	}

	intern = (php_framegrab_device_obj *)zend_object_store_get_object(getThis() TSRMLS_CC);
	if (NULL == (buffer = php_framegrab_device_data_to_string(&(intern->device)))) {
		RETURN_FALSE
	}

	ZVAL_STRING(return_value, buffer, 1)
	free(buffer);
	return;
}
/* }}} */

/* {{{ proto FrameGrabImage FrameGrabDevice::grab()
 * Grabs an image from the device
 */
PHP_METHOD(framegrabdevice, grab) {
	php_framegrab_device_obj *intern;
	php_framegrab_image_data image;
	php_framegrab_image_obj *intern_image;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "") == FAILURE) {
		RETURN_FALSE;
	}

	intern = (php_framegrab_device_obj *)zend_object_store_get_object(getThis() TSRMLS_CC);

	if (!php_framegrab_image_grab(intern->device.device, &image)) {
		RETURN_FALSE;
	}

	RETVAL_OBJECT(php_framegrab_image_sc_entry)
	intern_image = zend_object_store_get_object(return_value TSRMLS_CC);

	/* Just copy not clone since we don't want to duplicate the image data. */
	intern_image->image = image;
}
/* }}} */

/* {{{ */
static void framegrab_device_object_free(void *object TSRMLS_DC) {
	php_framegrab_device_obj *intern = (php_framegrab_device_obj *)object;

	/* Free up the device data */
	php_framegrab_device_data_destroy(&intern->device);

	zend_object_std_dtor(&intern->zo TSRMLS_CC);
	efree(object);
}
/* }}} */

/* {{{ */
static zend_object_value framegrab_device_object_new_ex(zend_class_entry *class_type, php_framegrab_device_obj **ptr TSRMLS_DC) {
	php_framegrab_device_obj *intern;
	zend_object_value retval;
	zval *tmp;

	intern = emalloc(sizeof(php_framegrab_device_obj));
	memset(intern, 0, sizeof(php_framegrab_device_obj));

	if (ptr) {
		*ptr = intern;
	}

	zend_object_std_init(&intern->zo, class_type TSRMLS_CC);
	zend_hash_copy(intern->zo.properties, &class_type->default_properties, (copy_ctor_func_t) zval_add_ref, (void *) &tmp, sizeof(zval *));

	retval.handle = zend_objects_store_put(intern, NULL, (zend_objects_free_object_storage_t) framegrab_device_object_free, NULL TSRMLS_CC);
	retval.handlers = (zend_object_handlers *) &framegrab_device_object_handlers;
	return retval;
}
/* }}} */

/* {{{ */
static zend_object_value framegrab_device_object_new(zend_class_entry *class_type TSRMLS_DC) {
	return framegrab_device_object_new_ex(class_type, NULL TSRMLS_CC);
}
/* }}} */

/* {{{ */
static zend_object_value framegrab_device_object_clone(zval *this_ptr TSRMLS_DC) {
	php_framegrab_device_obj *new_obj = NULL;
	php_framegrab_device_obj *old_obj = (php_framegrab_device_obj *)zend_object_store_get_object(this_ptr TSRMLS_CC);
	zend_object_value        retval = framegrab_device_object_new_ex(old_obj->zo.ce, &new_obj TSRMLS_CC);

	zend_objects_clone_members(&new_obj->zo, retval, &old_obj->zo, Z_OBJ_HANDLE_P(this_ptr) TSRMLS_CC);

	php_framegrab_device_data_init(&new_obj->device);
	php_framegrab_device_data_clone(&old_obj->device, &new_obj->device);

	return retval;
}
/* }}} */

#define FRAMEGRAB_DEVICE_PROP_NAME_DEVICE              "device"
#define FRAMEGRAB_DEVICE_PROP_NAME_LEN_DEVICE          6
#define FRAMEGRAB_DEVICE_PROP_NAME_CARD                "card"
#define FRAMEGRAB_DEVICE_PROP_NAME_LEN_CARD            4
#define FRAMEGRAB_DEVICE_PROP_NAME_DRIVER              "driver"
#define FRAMEGRAB_DEVICE_PROP_NAME_LEN_DRIVER          6
#define FRAMEGRAB_DEVICE_PROP_NAME_LEFT                "left"
#define FRAMEGRAB_DEVICE_PROP_NAME_LEN_LEFT            4
#define FRAMEGRAB_DEVICE_PROP_NAME_TOP                 "top"
#define FRAMEGRAB_DEVICE_PROP_NAME_LEN_TOP             3
#define FRAMEGRAB_DEVICE_PROP_NAME_WIDTH               "width"
#define FRAMEGRAB_DEVICE_PROP_NAME_LEN_WIDTH           5
#define FRAMEGRAB_DEVICE_PROP_NAME_HEIGHT              "height"
#define FRAMEGRAB_DEVICE_PROP_NAME_LEN_HEIGHT          6

/* {{{ */
static zval *framegrab_device_object_read_property(zval *object, zval *member, int type TSRMLS_DC) {
	php_framegrab_device_obj *intern;
	zval *retval;
	MAKE_STD_ZVAL(retval);

	intern = (php_framegrab_device_obj *)zend_object_store_get_object(object TSRMLS_CC);
	if (Z_TYPE_P(member) != IS_STRING) {
		ZVAL_NULL(retval);
	} else {
		char *prop_name   = Z_STRVAL_P(member);
		int prop_name_len = Z_STRLEN_P(member);

		if ((FRAMEGRAB_DEVICE_PROP_NAME_LEN_DEVICE == prop_name_len) &&
			!memcmp(prop_name, FRAMEGRAB_DEVICE_PROP_NAME_DEVICE, FRAMEGRAB_DEVICE_PROP_NAME_LEN_DEVICE)) {
			ZVAL_STRING(retval, intern->device.device, 1);
		} else
		if ((FRAMEGRAB_DEVICE_PROP_NAME_LEN_CARD == prop_name_len) &&
			!memcmp(prop_name, FRAMEGRAB_DEVICE_PROP_NAME_CARD, FRAMEGRAB_DEVICE_PROP_NAME_LEN_CARD)) {
			if (intern->device.card) {
				ZVAL_STRING(retval, intern->device.card, 1);
			} else {
				ZVAL_NULL(retval);
			}
		} else
		if ((FRAMEGRAB_DEVICE_PROP_NAME_LEN_DRIVER == prop_name_len) &&
			!memcmp(prop_name, FRAMEGRAB_DEVICE_PROP_NAME_DRIVER, FRAMEGRAB_DEVICE_PROP_NAME_LEN_DRIVER)) {
			if (intern->device.driver_name) {
				ZVAL_STRING(retval, intern->device.driver_name, 1);
			} else {
				ZVAL_NULL(retval);
			}
		} else
		if ((FRAMEGRAB_DEVICE_PROP_NAME_LEN_LEFT == prop_name_len) &&
			!memcmp(prop_name, FRAMEGRAB_DEVICE_PROP_NAME_LEFT, FRAMEGRAB_DEVICE_PROP_NAME_LEN_LEFT)) {
			ZVAL_LONG(retval, intern->device.def_left);
		} else
		if ((FRAMEGRAB_DEVICE_PROP_NAME_LEN_TOP == prop_name_len) &&
			!memcmp(prop_name, FRAMEGRAB_DEVICE_PROP_NAME_TOP, FRAMEGRAB_DEVICE_PROP_NAME_LEN_TOP)) {
			ZVAL_LONG(retval, intern->device.def_top);
		} else
		if ((FRAMEGRAB_DEVICE_PROP_NAME_LEN_WIDTH == prop_name_len) &&
			!memcmp(prop_name, FRAMEGRAB_DEVICE_PROP_NAME_WIDTH, FRAMEGRAB_DEVICE_PROP_NAME_LEN_WIDTH)) {
			ZVAL_LONG(retval, intern->device.def_width);
		} else
		if ((FRAMEGRAB_DEVICE_PROP_NAME_LEN_HEIGHT == prop_name_len) &&
			!memcmp(prop_name, FRAMEGRAB_DEVICE_PROP_NAME_HEIGHT, FRAMEGRAB_DEVICE_PROP_NAME_LEN_HEIGHT)) {
			ZVAL_LONG(retval, intern->device.def_height);
		} else {
			ZVAL_NULL(retval);
		}
	}

	Z_SET_REFCOUNT_P(retval, 0);
	return retval;
}
/* }}} */

static zend_function_entry framegrab_device_funcs[] = {
	PHP_ME(framegrabdevice, __construct, NULL, ZEND_ACC_CTOR|ZEND_ACC_PUBLIC)
	PHP_ME(framegrabdevice, grab, NULL, ZEND_ACC_PUBLIC)
	PHP_ME(framegrabdevice, __toString, NULL, ZEND_ACC_PUBLIC)

	/* End of functions */
	{NULL, NULL, NULL}
};


static zend_function_entry framegrab_funcs[] = {
	/* End of functions */
	{NULL, NULL, NULL}
};

/* {{{ PHP_MINIT_FUNCTION */
PHP_MINIT_FUNCTION(framegrab)
{
	zend_class_entry ce;

	/* Set up the object handlers */
	memcpy(&framegrab_device_object_handlers, zend_get_std_object_handlers(), sizeof(zend_object_handlers));
	memcpy(&framegrab_image_object_handlers, zend_get_std_object_handlers(), sizeof(zend_object_handlers));

	/*
	 *
	 */
	INIT_CLASS_ENTRY(ce, "FrameGrabDeviceException", NULL);
	php_framegrab_device_exception_class_entry = zend_register_internal_class_ex(&ce, zend_exception_get_default(TSRMLS_C), NULL TSRMLS_CC);
	php_framegrab_device_exception_class_entry->ce_flags |= ZEND_ACC_FINAL;

	/*
	 * FrameGrabDevice class
	 */
	INIT_CLASS_ENTRY(ce, "FrameGrabDevice", framegrab_device_funcs);
	ce.create_object = framegrab_device_object_new;
	framegrab_device_object_handlers.clone_obj = framegrab_device_object_clone;
	framegrab_device_object_handlers.read_property = framegrab_device_object_read_property;
	php_framegrab_device_sc_entry = zend_register_internal_class(&ce TSRMLS_CC);

	/*
	 * FrameGrabImage class
	 */
	INIT_CLASS_ENTRY(ce, "FrameGrabImage", framegrab_image_funcs);
	ce.create_object = framegrab_image_object_new;
	framegrab_image_object_handlers.clone_obj = framegrab_image_object_clone;
	framegrab_image_object_handlers.read_property = framegrab_image_object_read_property;
	php_framegrab_image_sc_entry = zend_register_internal_class(&ce TSRMLS_CC);


	return SUCCESS;
}
/* }}} */

/* {{{ PHP_MINFO_FUNCTION
 */
PHP_MINFO_FUNCTION(framegrab)
{
	php_info_print_table_start();
	php_info_print_table_header(2, "framegrab support", "enabled");
	php_info_print_table_row(2, "version", PHP_FRAMEGRAB_VERSION);
	php_info_print_table_end();
}
/* }}} */

zend_module_entry framegrab_module_entry = {
	STANDARD_MODULE_HEADER,
	"framegrab",
	framegrab_funcs,
	PHP_MINIT(framegrab),
	NULL,
	NULL,
	NULL,
	PHP_MINFO(framegrab),
	PHP_FRAMEGRAB_VERSION,
	STANDARD_MODULE_PROPERTIES
};

//#ifdef COMPILE_DL_FRAMEGRAB
ZEND_GET_MODULE(framegrab)
//#endif

/*
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * End:
 * vim600: fdm=marker
 * vim: noet sw=4 ts=4
 */

