/*
   +----------------------------------------------------------------------+
   | PHP Version 5                                                        |
   +----------------------------------------------------------------------+
   | Copyright (c) 1997-2010 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
   | available through the world-wide-web at the following url:           |
   | http://www.php.net/license/3_01.txt                                  |
   | If you did not receive a copy of the PHP license and are unable to   |
   | obtain it through the world-wide-web, please send a note to          |
   | license@php.net so we can mail you a copy immediately.               |
   +----------------------------------------------------------------------+
   | Author: Melanie Rhianna Lewis <cyberspice@php.net>                   |
   +----------------------------------------------------------------------+
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "php_framegrab_device.h"

#define FRAMEGRAB_DEVICE_STRING_BUFFER_SIZE 1024

/**
 * Initialises a php_framegrab_device_data structure.
 */
void php_framegrab_device_data_init(php_framegrab_device_data *data) {
	data->device      = NULL;
	data->driver_name = NULL;
	data->card        = NULL;
	data->def_left    = 0;
	data->def_top     = 0;
	data->def_width   = 0;
	data->def_height  = 0;
}

/**
 * Initialises a php_framegrab_device_data structure with the specified
 * values.
 */
int php_framegrab_device_data_init_ex(php_framegrab_device_data *data,
		                              int left, int top, int width, int height,
		                              char *device, char *driver_name, char *card) {
	data->def_left   = left;
	data->def_top    = top;
	data->def_width  = width;
	data->def_height = height;

	if (!device) {
		return 0;
	}

	if (NULL == (data->device = (char*)malloc(strlen(device) + 1))) {
		return 0;
	}
	strcpy(data->device, device);

	if (driver_name) {
		if (NULL == (data->driver_name = (char*)malloc(strlen(driver_name) + 1))) {
			php_framegrab_device_data_destroy(data);
			return 0;
		}
		strcpy(data->driver_name, driver_name);
	} else {
		data->driver_name = NULL;
	}

	if (card) {
		if (NULL == (data->card = (char*)malloc(strlen(card) + 1))) {
			php_framegrab_device_data_destroy(data);
			return 0;
		}
		strcpy(data->card, card);
	} else {
		data->card = NULL;
	}

	return 1;
}

/**
 * Destroys a php_framegrab_device_data structure freeing any memory
 * it is using.
 */
void php_framegrab_device_data_destroy(php_framegrab_device_data *data) {
	if (data->device) {
		free(data->device);
	}

	if (data->driver_name) {
		free(data->driver_name);
	}

	if (data->card) {
		free(data->card);
	}
}

/**
 * Clones a php_framegrab_device_data structure.
 */
int php_framegrab_device_data_clone(php_framegrab_device_data *data,
		                            php_framegrab_device_data *copydata) {
	return php_framegrab_device_data_init_ex(copydata,
			data->def_left, data->def_top, data->def_width, data->def_height,
			data->device, data->driver_name, data->card);
}

/**
 * Returns a character string describing the device.
 */
char *php_framegrab_device_data_to_string(php_framegrab_device_data *data) {
	char *buffer = (char*)calloc(FRAMEGRAB_DEVICE_STRING_BUFFER_SIZE, sizeof(char));
	int len = FRAMEGRAB_DEVICE_STRING_BUFFER_SIZE;
	char *ptr = buffer;
	int size;

	size = snprintf(ptr, len,
			 "FrameGrabDevice:device=\"%s\",", data->device);
	len -= size;
	ptr += size;

	if (data->card) {
		size = snprintf(ptr, len, "card=\"%s\",", data->card);
		len -= size;
		ptr += size;
	}

	if (data->driver_name) {
		size = snprintf(ptr, len, "driver_name=\"%s\",", data->driver_name);
		len -= size;
		ptr += size;
	}

	snprintf(ptr, len, "left=%d,top=%d,width=%d,height=%d",
			 data->def_left, data->def_top, data->def_width, data->def_height);

	return buffer;
}

/*
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * indent-tabs-mode: t
 * End:
 */

